package imagecomparer.gui;

import imagecomparer.stream.DelimitedImageInputStream;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.File;

/**
 * Class:           imagecomparer.gui.ConfigPanel
 * Version:         0.2
 * Date:            2004-04-25 21:23:05
 *
 * Description:     Builds and operate the config panel.
 *
 * COPYRIGHT:
 *
 * This software module was originally developed by Pawel Halasa.
 * 
 */
public class ConfigPanel extends JPanel implements ActionListener, FileSelectListener {

    private JComboBox viewTypeComboBox, viewModeComboBox;
    private JTextField capacityField;
    private int capacity = 400;
    private JButton pauseButton, restartButton;
    private boolean processStarted = true, processPaused = false;

    public ConfigPanel() {
        this(null);
    }

    /**
     * Creates content of control panel. Automaticly sets capacity to 200 b/s, view type
     * to normal and view mode to normal.
     * */
    public ConfigPanel(File referenceFile) {
        setBorder(BorderFactory.createTitledBorder("Configuration Panel"));
        setPreferredSize(new Dimension(200, 300));

        Box box = Box.createVerticalBox();
        box.setAlignmentX(CENTER_ALIGNMENT);
        add(box);

        box.add(new JLabel("Original source:"));
        box.add(new FileSelectBox(this, 2, referenceFile));
        box.add(Box.createVerticalStrut(10));

        // View type: Normal | Scaled
        Box viewTypeBox = Box.createHorizontalBox();
        box.add(viewTypeBox);
        JLabel viewTypeLabel = new JLabel("View type:");
        viewTypeBox.add(viewTypeLabel);
        viewTypeBox.add(Box.createHorizontalStrut(5));
        String [] viewTypes = {"Normal", "Fitted"};
        viewTypeComboBox = new JComboBox(viewTypes);
        viewTypeBox.add(viewTypeComboBox);
        viewTypeComboBox.addActionListener(this);
        box.add(Box.createVerticalStrut(10));

        // View mode: Normal | Scaled
        Box viewModeBox = Box.createHorizontalBox();
        box.add(viewModeBox);
        JLabel viewModeLabel = new JLabel("View mode:");
        viewModeBox.add(viewModeLabel);
        viewModeBox.add(Box.createHorizontalStrut(5));
        String [] viewModes = {"Normal", "Difference"};
        viewModeComboBox = new JComboBox(viewModes);
        viewModeBox.add(viewModeComboBox);
        viewModeComboBox.addActionListener(this);
        box.add(Box.createVerticalStrut(10));

        // Speed
        Box speedBox = Box.createHorizontalBox();
        box.add(speedBox);
        JLabel s = new JLabel("Capacity [bps] : ");
        speedBox.add(s);
        speedBox.add(Box.createHorizontalStrut(5));
        capacityField = new JTextField("");
        DelimitedImageInputStream.setCapacity(capacity);
        capacityField.setText("" + capacity);
        capacityField.addActionListener(this);
        speedBox.add(capacityField);
        box.add(Box.createVerticalStrut(10));

        // Speed
        Box startBox = Box.createHorizontalBox();
        box.add(startBox);
        restartButton = new JButton((processStarted) ? "Restart" : "Start");
        restartButton.addActionListener(this);
        startBox.add(restartButton);
        startBox.add(Box.createHorizontalStrut(5));
        pauseButton = new JButton((processPaused) ? "Resume" : "Pause");
        pauseButton.addActionListener(this);
        startBox.add(pauseButton);
    }

    /**
     * Performs action handling when:
     * - view type has changed,
     * - view mode has changed,
     * - capacity has been changed,
     * - one of resume/pause buttons has been selected.
     *
     * @param e Action event
     * */
    public void actionPerformed(ActionEvent e) {
        Object o = e.getSource();

        if (o == viewTypeComboBox) {
            int i = viewTypeComboBox.getSelectedIndex();
            if (i == 0) {
                ImagePane.setScalling(false);
            } else {
                ImagePane.setScalling(true);
            }
        } else if (o == viewModeComboBox) {
            int i = viewModeComboBox.getSelectedIndex();
            if (i == 0) {
                ImagePane.setDifferenceMode(false);
            } else {
                ImagePane.setDifferenceMode(true);
            }
        } else if (o == capacityField) {
            try {
                capacity  = new Integer(capacityField.getText()).intValue();
                DelimitedImageInputStream.setCapacity(capacity);
            } catch (NumberFormatException nfe) {
                capacityField.setText("" + capacity);
            }
        } else if (o == pauseButton) {
            if (processPaused)
                DelimitedImageInputStream.resume();
            else
                DelimitedImageInputStream.pause();
            processPaused = !processPaused;
            pauseButton.setText((processPaused) ? "Resume" : "Pause");
        } else if (o == restartButton) {
            ImagePane.restart();
        }
    }

    /**
     * Set the reference file, to which other images will be compared
     *
     * @param file The reference file
     */ 
    public void fileSelected(File file) {
        ImagePane.setReferenceFile(file);
    }
}
