package imagecomparer.gui;

import imagecomparer.stream.DelimitedImageInputStream;

import javax.swing.*;
import javax.imageio.ImageIO;
import javax.imageio.ImageReader;
import javax.imageio.spi.ImageReaderSpi;
import javax.imageio.event.IIOReadUpdateListener;
import javax.imageio.event.IIOReadProgressListener;
import javax.imageio.stream.ImageInputStream;
import javax.imageio.stream.FileImageInputStream;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.File;
import java.util.Iterator;
import java.util.Vector;
import java.util.Arrays;

/**
 * Class:           imagecomparer.gui.ImagePane
 * Version:         0.3
 * Date:            2004-04-21 01:26:05
 * <p/>
 * Description:     Creates an image panel.
 * <p/>
 * COPYRIGHT:
 * <p/>
 * This software module was originally developed by Pawel Halasa.
 */
public class ImagePane extends JPanel implements Runnable, IIOReadUpdateListener, IIOReadProgressListener {
    private BufferedImage normalImg = null, diffImg = null;
    static private BufferedImage refImg = null;
    private ImageReader reader;
    private ImageReaderSpi readerSpi;
    private Thread thread;
    private int paneWidth, paneHeight;
    private DelimitedImageInputStream diis;
    int dstPix[] = new int[3];
    int srcPix[] = new int[3];
    int refPix[] = new int[3];
    private JTextArea info;

    // mean square error array
    private long[] mse = new long[101];

    // temporary mean square error
    private long tempMse;
    private boolean stateReady;

    private enum ViewType {
        normal, scalled
    };
    private static ViewType viewType = ViewType.normal;

    private enum ViewMode {
        normal, difference
    };
    private static ViewMode viewMode = ViewMode.normal;
    private static Vector imagePanes = new Vector();
    static private int width, height;
    private JProgressBar pb;

    /**
     * The constructor of ImagePane.
     *
     * @param image Image input stream
     * @param pb    The progress bar which is informed about image progression
     */
    public ImagePane(File image, JProgressBar pb, JTextArea info) {
        super();
        if (pb != null) this.pb = pb;
        if (info != null) this.info = info;
        imagePanes.add(this);
        paneWidth = 347;
        paneHeight = 297;
        setVisible(true);
        if (image != null) setImage(image);
    }

    private void print(String text) {
        if (info != null)
            info.append(text);
    }

    private void clearTextArea() {
        info.setText("");
    }

    /**
     * Sets the reference file with which other images will be compared.
     *
     * @param file The reference file
     */
    static public boolean setReferenceFile(File file) {
        if (file == null) {
            refImg = null;
            return false;
        }

        ImageInputStream in;
        try {
            in = new FileImageInputStream(file);
        } catch (IOException e) {
            return false;
        }
        Iterator iter = ImageIO.getImageReaders(in);
        ImageReader r;
        if (iter.hasNext()) {
            r = (ImageReader) iter.next();
        } else {
            return false;
        }

        r.setInput(in);
        try {
            refImg = r.read(0);
        } catch (IOException e) {
            refImg = null;
            return false;
        }
        return true;
    }

    /**
     * Updates all image panes.
     */
    private static void updateAllPanes() {
        for (int i = 0; i < imagePanes.size(); i++) {
            ImagePane ip = (ImagePane) imagePanes.get(i);
            ip.updatePane();
        }
    }

    /**
     * Updates current image pane.
     */
    private void updatePane() {
        if (normalImg == null)
            setPreferredSize(new Dimension(0, 0));
        else {
            Dimension newSize = (viewType == ViewType.scalled) ?
                new Dimension(paneWidth, paneHeight) : new Dimension(width, height);
            if (newSize != getSize())
                setPreferredSize(newSize);
        }
        revalidate();
        repaint();
    }

    /**
     * Sets the difference mode, in which only ifferences between loading picturus
     * and reference picture will be displayed.
     *
     * @param mode If true the difference mode is set
     */
    public static void setDifferenceMode(boolean mode) {
        viewMode = (mode) ? ViewMode.difference : ViewMode.normal;
        updateAllPanes();
    }

    /**
     * Defines if displayed image should be fited to panel size.
     *
     * @param scalling If true the image will be fited to panel size.
     */
    static public void setScalling(boolean scalling) {
        viewType = (scalling) ? ViewType.scalled : ViewType.normal;
        updateAllPanes();
    }

    /**
     * Indicates whether difference mode is set.
     *
     * @return True if difference mode is set, false if not.
     */
    public static boolean isDifferenceMode() {
        return (viewMode == ViewMode.difference) ? true : false;
    }

    /**
     * Stops current images loading processes and starts them from the begining
     */
    public static void restart() {
        for (int i = 0; i < imagePanes.size(); i++)
            ((ImagePane) imagePanes.get(i)).clearPane();
        int cap = DelimitedImageInputStream.getCapacity();
        DelimitedImageInputStream.setCapacity(Integer.MAX_VALUE);
        for (int i = 0; i < imagePanes.size(); i++)
            try {
                ((ImagePane) imagePanes.get(i)).thread.join();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        DelimitedImageInputStream.setCapacity(cap);
        for (int i = 0; i < imagePanes.size(); i++)
            ((ImagePane) imagePanes.get(i)).clearPane();
        for (int i = 0; i < imagePanes.size(); i++)
            ((ImagePane) imagePanes.get(i)).prepareThread();
        for (int i = 0; i < imagePanes.size(); i++)
            ((ImagePane) imagePanes.get(i)).startThread();
    }

    /**
     * Sets a new image and starts loading process.
     *
     * @param image The image input stream
     */
    public synchronized void setImage(File image) {
        clearPane();
        reader = null;

        if (image == null) {
            return;
        }
        // Create an image input stream on the image
        ImageInputStream iis = null;
        try {
            iis = ImageIO.createImageInputStream(image);
            diis = new DelimitedImageInputStream(iis);
        } catch (IOException e) {
            JOptionPane.showMessageDialog(this, "Cannot open file " + image.getPath(),
                    "Loading Error", JOptionPane.ERROR_MESSAGE);
            return;
        }

        // Find all image readers that recognize the image format
        Iterator iter = ImageIO.getImageReaders(iis);
        if (iter.hasNext()) {
            readerSpi = ((ImageReader) iter.next()).getOriginatingProvider();
        } else {
            JOptionPane.showMessageDialog(this, "Cannot recognize file format",
                    "Loading Error", JOptionPane.ERROR_MESSAGE);
            return;
        }

        prepareThread();
        startThread();
    }

    private synchronized void clearPane() {
        if (reader != null) reader.abort();
        clearTextArea();
        pb.setValue(0);
        pb.setString("");
        updatePane();
        stateReady = false;
    }

    private void prepareThread() {
        try {
            diis.restartSteam();
        } catch (IOException e) {
            e.printStackTrace(); 
        }
        try {
            reader = readerSpi.createReaderInstance();
        } catch (IOException e) {
            e.printStackTrace();
        }
        reader.setInput(diis);
        reader.addIIOReadUpdateListener(this);
        reader.addIIOReadProgressListener(this);

        thread = new Thread(this);
        //thread.setPriority(Thread.MIN_PRIORITY + 2);
        normalImg = null;
        diffImg = null;
    }

    private void startThread() {
        thread.start();
    }

    public void run() {

        print("Start reading file header...\n");
        try {
            width = reader.getWidth(0);
            height = reader.getHeight(0);
        } catch (Exception e) {
            JOptionPane.showMessageDialog(this, "Cannot read image size",
                    "Loading Error", JOptionPane.ERROR_MESSAGE);
            return;
        }
        print("Image dimensions: " + width + " x " + height + "\n");

        normalImg = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
        Graphics g = normalImg.getGraphics();
        g.setColor(new Color(0, 0, 0));
        g.fillRect(0, 0, width, height);

        Arrays.fill(mse, -1);
        diffImg = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
        imageUpdate(reader, normalImg, 0, 0, width, height, 1, 1, null);
        updatePane();

        print("Start loading file...\n");
        imageProgress(reader, 0);

        // wait for another image
        stateReady = true;
        boolean state = false;
        while (!state) {
            state = true;
            for (int i = 0; i < imagePanes.size(); i++)
                state = state && ((ImagePane) imagePanes.get(i)).stateReady;
        }

        try {
            reader.read(0);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        BufferedImage img;

        if (normalImg == null) {
            g.setColor(new Color(200, 200, 200));
            g.fillRect(0, 0, paneWidth, paneHeight);
            return;
        }

        if (viewMode == ViewMode.difference) {
            img = diffImg;
        } else
            img = normalImg;

        int imgWidth = img.getWidth();
        int imgHeight = img.getHeight();

        if (viewType == ViewType.scalled) {
            double m;
            if ((double) imgWidth / imgHeight > (double) paneWidth / paneHeight)
                m = (double) paneWidth / imgWidth;
            else
                m = (double) paneHeight / imgHeight;
            g.drawImage(img, 0, 0, (int) (imgWidth * m), (int) (imgHeight * m), this);
        } else {
            g.drawImage(img, 0, 0, this);
        }
    }

    public void passStarted(ImageReader source, BufferedImage theImage, int pass, int minPass, int maxPass, int minX, int minY, int periodX, int periodY, int[] bands) {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    public void imageUpdate(ImageReader source, BufferedImage srcImg, int minX, int minY, int width, int height, int periodX, int periodY, int[] bands) {
        if (source != reader) return;
        if (srcImg == null) return;

        if (refImg == null) {
            Graphics g = diffImg.getGraphics();
            g.setColor(new Color(255, 0, 0));
            g.fillRect(0, 0, diffImg.getWidth(this), diffImg.getHeight(this));
            g.setColor(new Color(255, 255, 255));
            g.drawString("The reference image hasn't been selected!", 10, 15);
        } else {
            Color srcCol, refCol;
            int r, g, b;
            for (int y = minY; y < minY+height; y++)
                for (int x = minX; x < minX+width; x++) {
                    srcCol = new Color(srcImg.getRGB(x, y));
                    refCol = new Color(refImg.getRGB(x, y));
                    r = Math.abs(srcCol.getRed() - refCol.getRed());
                    g = Math.abs(srcCol.getGreen() - refCol.getGreen());
                    b = Math.abs(srcCol.getBlue() - refCol.getBlue());
                    diffImg.setRGB(x, y, new Color(r, g, b).getRGB());
                }
        }

        normalImg = srcImg;
        repaint();
    }

    public void passComplete(ImageReader source, BufferedImage theImage) {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    public void thumbnailPassStarted(ImageReader source, BufferedImage theThumbnail, int pass, int minPass, int maxPass, int minX, int minY, int periodX, int periodY, int[] bands) {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    public void thumbnailUpdate(ImageReader source, BufferedImage theThumbnail, int minX, int minY, int width, int height, int periodX, int periodY, int[] bands) {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    public void thumbnailPassComplete(ImageReader source, BufferedImage theThumbnail) {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    public void imageComplete(ImageReader source) {
        if (source != reader) return;
        imageProgress(source, 100);
        print("Image complete.\n");
    }

    public void readAborted(ImageReader source) {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    public void sequenceComplete(ImageReader source) {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    public void thumbnailComplete(ImageReader source) {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    public void imageProgress(ImageReader source, float percentageDone) {
        int perc = (int) percentageDone;
        if (source != reader) return;
        if (pb == null) return;
        pb.setValue(perc);
        pb.setString("" + perc + "%");
        if ((refImg != null) && (mse[perc] == -1)) {
            Color diffCol;
            int r, g, b;
            tempMse = 0;
            for (int y = 0; y < diffImg.getHeight(); y++)
                for (int x = 0; x < diffImg.getWidth(); x++) {
                    diffCol = new Color(diffImg.getRGB(x, y));
                    r = diffCol.getRed();
                    g = diffCol.getGreen();
                    b = diffCol.getBlue();
                    tempMse += (r*r) + (g*g) + (b*b);
                }
            tempMse /= 3 * diffImg.getHeight() * diffImg.getWidth();

            mse[(int)percentageDone] = tempMse;
            print("MSE after " + (int)percentageDone + "% = " + tempMse + "\n");
        }
    }

    public void thumbnailProgress(ImageReader source, float percentageDone) {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    public void imageStarted(ImageReader source, int imageIndex) {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    public void sequenceStarted(ImageReader source, int minIndex) {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    public void thumbnailStarted(ImageReader source, int imageIndex, int thumbnailIndex) {
        //To change body of implemented methods use File | Settings | File Templates.
    }

}
